#!/bin/sh
# Script for backing up mysql databases.
# Databases to be backed up.
export DATABASES='vmail mysql roundcubemail'
# Database character set for ALL databases.
# Note: Currently, it doesn't support to specify character set for each databases.
export DB_CHARACTER_SET="utf8"
# Commands
export CMD_MYSQLDUMP="mysqldump --events --ignore-table=mysql.event --default-character-set=${DB_CHARACTER_SET} --skip-comments"
export CMD_DU='du -sh'
# Pre-defined backup status
export BACKUP_SUCCESS='NO'
# Date
export YEAR="$(date +"%Y")"
export MONTH="$(date +"%m")"
export DAY="$(date +"%d")"
export TIME="$(date +"%H:%M:%S")"
export TIMESTAMP="${YEAR}-${MONTH}-${DAY}-${TIME}"
# Location of mysql backup root directory.
export BACKUP_ROOTDIR="/var/vmail/backup/mysql"
# Define, check, create directories.
export BACKUP_DIR="${BACKUP_ROOTDIR}/${YEAR}/${MONTH}/${DAY}"
# Log file
export LOGFILE="${BACKUP_DIR}/${TIMESTAMP}.log"
#Check and create directories.
[ ! -d ${BACKUP_DIR} ] && mkdir -p ${BACKUP_DIR} 2>/dev/null
chown root ${BACKUP_DIR}
chmod 0400 ${BACKUP_DIR}
# Initialize log file.
echo "* Starting backup: ${TIMESTAMP}." >${LOGFILE}
echo "* Backup directory: ${BACKUP_DIR}." >>${LOGFILE}
chmod 0400 ${LOGFILE}
# Begin backup of mysql databases.
echo "* Backing up databases: ${DATABASES}." >> ${LOGFILE}
if ! ((${CMD_MYSQLDUMP} --databases vmail > /${BACKUP_DIR}/${TIMESTAMP}-vmail.sql) && 
     (${CMD_MYSQLDUMP} --databases mysql > /${BACKUP_DIR}/${TIMESTAMP}-mysql.sql) && 
     (${CMD_MYSQLDUMP} --databases roundcubemail > /${BACKUP_DIR}/${TIMESTAMP}-roundcubemail.sql));
then 
echo "* Backup completed (Success? ${BACKUP_SUCCESS})." >>${LOGFILE}
cat $LOGFILE | mail -s "MySQL Databases Backup" root
exit
else
export BACKUP_SUCCESS='YES'
chown -R root ${BACKUP_DIR}
chmod -R 0400 ${BACKUP_DIR}
# Append file size of backup files.
echo -e "* File size:\n----" >>${LOGFILE}
cd ${BACKUP_DIR} && \
${CMD_DU} *${TIMESTAMP}*sql* >>${LOGFILE}
echo "----" >>${LOGFILE}
echo "* Backup completed (Success? ${BACKUP_SUCCESS})." >>${LOGFILE}
cat $LOGFILE | mail -s "MySQL Databases Backup" root

if !(grep $BACKUP_DIR /var/vmail/backup/daycount); then
echo "$BACKUP_DIR" >> /var/vmail/backup/daycount
fi

fi

# Code below deletes old logs.
# Adjust NUMDAYS to your liking.
NUMDAYS=30
BASEDIR="/var/vmail/backup/mysql"
COUNTFILE="/var/vmail/backup/daycount"
LOGHEAD=$(head -n1 $COUNTFILE)
LINECOUNT=$(wc -l < $COUNTFILE)
NEWCOUNT=$(tail -n +2 $COUNTFILE)

if (($LINECOUNT > $NUMDAYS)); then
rm -rf $LOGHEAD
echo "$NEWCOUNT" > $COUNTFILE
fi

find $BASEDIR/* -type d -empty -exec rm -rf {} \; 2>/dev/null
